package com.baidu.appx.adentity;

import java.io.IOException;
import java.io.Serializable;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.baidu.appx.adinfo.AppWallAdInfo;
import com.baidu.appx.adinfo.BannerAdInfo;
import com.baidu.appx.adinfo.ClickableAd;
import com.baidu.appx.adinfo.InterstitialAdInfo;
import com.baidu.appx.adinfo.NativeAdInfo;
import com.baidu.appx.adinfo.SplashAdInfo;
import com.baidu.appx.util.DownloadImageMng;
import com.baidu.appx.util.DownloadImageMng.DownloadImageListener;

public class PlatformAdInfo extends BaseAdInfo implements SplashAdInfo,
BannerAdInfo.All, InterstitialAdInfo, NativeAdInfo, AppWallAdInfo, Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5526493016453821449L;

	public PlatformAdInfo(JSONObject json) {
		super(json);
	}
	
	public int    	getAdLayoutType() 	{return intValueForKey("admt");}
	public String 	getAdHtml() 		{return stringValueForKey("adm");}
	public String 	getAdImgUrl() 		{return stringValueForKey("adi");}
	public String 	getAdIconUrl() 		{return stringValueForKey("adicon");}
	public String 	getAdTitle()		{return stringValueForKey("adt");}
	public String 	getAdSubTitle() 	{return stringValueForKey("ads");}
	public String 	getAdBgImgUrl() 	{return stringValueForKey("bgimage");}
	public String 	getAdTextColor() 	{return stringValueForKey("textcolor");}
	public String 	getAdBgColor() 		{return stringValueForKey("bgcolor");}
	public long    	getAdRefreshMilliseconds() 	{return longValueForKey("rm");}
	public long    	getAdCachedSeconds() 		{return longValueForKey("openadcachesec");}
	
	public int    	getAdAnimatingDirection() 	{return intValueForKey("ad_direction");}
	public JSONArray getDisplayNotifyUrls() 	{return json.optJSONArray("iurl");}
	public JSONArray getClickNotifyUrls() 		{return json.optJSONArray("curl");}
	public JSONArray getDownloadNotifyUrls()	{return json.optJSONArray("durl");}
	public String 	getAdLinkedUrl() 	{return stringValueForKey("adcurl");}
	public int    	getClickType() 		{return intValueForKey("adct");}
	public String 	getRequestId() 		{return stringValueForKey("requestid");}
	public int    	getBalanceType() 	{return intValueForKey("st");} // not used
	public String 	getApkMd5() 		{return stringValueForKey("MD5");}
	public String 	getApkIdentifier() 	{return stringValueForKey("appidentifier");}
	public boolean 	getShouldConfirmWhenClick() 	{return boolValueForKey("ft");}
	public int    	getAdType() 		{return intValueForKey("attr");}
	public String 	getAppRating()		{return stringValueForKey("rating");}
	public String 	getDownloadNum()	{return stringValueForKey("downloadnum");}
	public String 	getApkSize()		{return stringValueForKey("filesize");}
	public String 	getAdTag()			{return stringValueForKey("adid");}
	public int	 	getAdCategory()		{return intValueForKey("cat");}
	public boolean 	getShouldDiscount()	{return boolValueForKey("isshowrs");}
	public boolean 	getEnableIgnore()	{return boolValueForKey("isshowskip");}
	
	public int 		getIconHeight()		{return intValueForKey("adiconh");}
	public int 		getIconWidth()		{return intValueForKey("adiconw");}
	public int 		getImageHeight()	{return intValueForKey("adih");}
	public int 		getImageWidth()		{return intValueForKey("adiw");}
	
	public String getAdBgImgPath()	 	{return pathOfBgImg;}
	public String getAdImgPath()	 	{return pathOfAdImg;}
	public String getAdIconPath() 		{return pathOfAdIcon;}
	
	public BannerAdInfo.Img 		getLayoutImg() 		{return this;}
	public BannerAdInfo.Text  		getLayoutText() 	{return this;}
	public BannerAdInfo.ImgText 	getLayoutImgText()	{return this;}
	public BannerAdInfo.Html  		getLayoutHtml() 	{return this;}
	
	public String pathOfBgImg 	= "";
	public String pathOfAdImg 	= "";
	public String pathOfAdIcon 	= "";
	
	
	public static final int ImgUrlTagAd 	= 1;
	public static final int ImgUrlTagBg 	= 2;
	public static final int ImgUrlTagIcon 	= 3;

	public int cachedImgCount = 0; //reserved
	
	public interface AdImgLoaderDelegate {
		public void adImgDidLoadWithUrlTag(int urlTag, PlatformAdInfo plfmAd, Object extendedObj);
	}
	
	public void downloadImgWithUrlTag(final int urlTag, final String imgPrefix, final AdImgLoaderDelegate delegate, final Object extendedObj) {
		String stringUrl = "";
		switch (urlTag) {
		case ImgUrlTagAd:
			stringUrl = getAdImgUrl();
			break;
		case ImgUrlTagBg:
			stringUrl = getAdBgImgUrl();
			break;
		case ImgUrlTagIcon:
			stringUrl = getAdIconUrl();
			break;
		}

		DownloadImageMng.getInstance().addDownloadDuty(stringUrl, imgPrefix, new DownloadImageListener() {

			@Override
			public void onDownloadImageFinish(Error err, String url, String filePath) {
				switch (urlTag) {
				case ImgUrlTagAd:
					PlatformAdInfo.this.pathOfAdImg = filePath;
					break;
				case ImgUrlTagBg:
					PlatformAdInfo.this.pathOfBgImg = filePath;
					break;
				case ImgUrlTagIcon:
					PlatformAdInfo.this.pathOfAdIcon = filePath;
					break;
				}
				delegate.adImgDidLoadWithUrlTag(urlTag, PlatformAdInfo.this, extendedObj);
			}
		});
	}
	
	public boolean 	 isValidClicableAd() {
		switch (getClickType()) {
		case ClickableAd.ClickToDownloadApp:
			return !getAdLinkedUrl().isEmpty() && !getApkIdentifier().isEmpty();
		case ClickToOpenWebview:
			return !getAdLinkedUrl().isEmpty();
		default:
			return true;
		}
	}
	
	
	private void writeObject(java.io.ObjectOutputStream ou)throws IOException {
		ou.writeUTF(pathOfAdIcon);
		ou.writeUTF(pathOfAdImg);
		ou.writeUTF(pathOfBgImg);
		ou.writeUTF(json.toString());
		ou.writeInt(cachedImgCount);
	}
	private void readObject(java.io.ObjectInputStream in)throws IOException,ClassNotFoundException, JSONException {
		pathOfAdIcon = in.readUTF();
		pathOfAdImg = in.readUTF();
		pathOfBgImg = in.readUTF();
		String jsonStr = in.readUTF();
		json = new JSONObject(jsonStr);
		cachedImgCount = in.readInt();
	}
}

